@@ -5,6 +5,7 @@ import unittest
 
 import sys
 sys.path.insert(0, "../")
+from sys import getrefcount
 
 import cairo
 
@@ -46,6 +47,37 @@ class TestEverything(unittest.TestCase):
         self.assertEquals(surface.get_width(), 10)
         self.assertEquals(surface.get_height(), 10)
 
+    def test_floating(self):
+        Everything.TestFloating()
+
+    def test_caller_allocates(self):
+        struct_a = Everything.TestStructA()
+        struct_a.some_int = 10
+        struct_a.some_int8 = 21
+        struct_a.some_double = 3.14
+        struct_a.some_enum = Everything.TestEnum.VALUE3
+
+        struct_a_clone = struct_a.clone()
+        self.assertTrue(struct_a != struct_a_clone)
+        self.assertEquals(struct_a.some_int, struct_a_clone.some_int)
+        self.assertEquals(struct_a.some_int8, struct_a_clone.some_int8)
+        self.assertEquals(struct_a.some_double, struct_a_clone.some_double)
+        self.assertEquals(struct_a.some_enum, struct_a_clone.some_enum)
+
+        struct_b = Everything.TestStructB()
+        struct_b.some_int8 = 8
+        struct_b.nested_a.some_int = 20
+        struct_b.nested_a.some_int8 = 12
+        struct_b.nested_a.some_double = 333.3333
+        struct_b.nested_a.some_enum = Everything.TestEnum.VALUE2
+
+        struct_b_clone = struct_b.clone()
+        self.assertTrue(struct_b != struct_b_clone)
+        self.assertEquals(struct_b.some_int8, struct_b_clone.some_int8)
+        self.assertEquals(struct_b.nested_a.some_int, struct_b_clone.nested_a.some_int)
+        self.assertEquals(struct_b.nested_a.some_int8, struct_b_clone.nested_a.some_int8)
+        self.assertEquals(struct_b.nested_a.some_double, struct_b_clone.nested_a.some_double)
+        self.assertEquals(struct_b.nested_a.some_enum, struct_b_clone.nested_a.some_enum)
 
 class TestNullableArgs(unittest.TestCase):
     def test_in_nullable_hash(self):
@@ -56,24 +88,30 @@ class TestNullableArgs(unittest.TestCase):
         Everything.test_glist_null_in(None)
 
     def test_in_nullable_array(self):
-        Everything.test_array_int_null_in(None, -1)
+        Everything.test_array_int_null_in(None)
 
     def test_in_nullable_string(self):
         Everything.test_utf8_null_in(None)
 
+    def test_in_nullable_object(self):
+        Everything.test_object_null_in(None)
+
     def test_out_nullable_hash(self):
         self.assertEqual(None, Everything.test_ghash_null_out())
 
     def test_out_nullable_list(self):
-        self.assertEqual(None, Everything.test_gslist_null_out())
-        self.assertEqual(None, Everything.test_glist_null_out())
+        self.assertEqual([], Everything.test_gslist_null_out())
+        self.assertEqual([], Everything.test_glist_null_out())
 
     def test_out_nullable_array(self):
-        self.assertEqual((None, 0), Everything.test_array_int_null_out())
+        self.assertEqual(None, Everything.test_array_int_null_out())
 
     def test_out_nullable_string(self):
         self.assertEqual(None, Everything.test_utf8_null_out())
 
+    def test_out_nullable_object(self):
+        self.assertEqual(None, Everything.test_object_null_out())
+
 class TestCallbacks(unittest.TestCase):
     called = False
     main_loop = GObject.MainLoop()
@@ -159,6 +197,23 @@ class TestCallbacks(unittest.TestCase):
             
         self.assertEquals(TestCallbacks.called, 100)
 
+    def testCallbackUserdataRefCount(self):
+        TestCallbacks.called = False
+        def callback(userdata):
+            TestCallbacks.called = True
+            return 1
+
+        ud = "Test User Data"
+
+        start_ref_count = getrefcount(ud)
+        for i in range(100):
+            Everything.test_callback_destroy_notify(callback, ud)
+
+        Everything.test_callback_thaw_notifications()
+        end_ref_count = getrefcount(ud)
+
+        self.assertEquals(start_ref_count, end_ref_count)
+
     def testAsyncReadyCallback(self):
         TestCallbacks.called = False
         TestCallbacks.main_loop = GObject.MainLoop()
